#ifndef _LABGEN_H
#define _LABGEN_H

#define TAILLE 255

typedef struct {
int c;
int l;
} Point;

typedef struct Point* point_ptr;

typedef struct {
char* ident[TAILLE];
int val[TAILLE];
int last;
} variables;

typedef struct {
char* ident[TAILLE];
Point* val[TAILLE];
int last;
} varpoint;

/******************************************************************************/
/* Fonctions de création de structure de points                               */
/******************************************************************************/

/* [new_point(c,l)] crée un nouveau point de coordonnées (c,l) */
Point new_point(int ,int);
/* [new_point_ptr(c,l)] crée un nouveau pointeur de point de coordonnées (c,l) */
Point* new_point_ptr(int, int);


/******************************************************************************/
/* Fonctions de base de matrices                                              */
/******************************************************************************/

/* [create_matrice(c,l)] crée une matrice à c colonnes et l lignes */
char*** create_matrice(int,int);
/* [lecture(p,matrice)] renvoie la valeur de la matrice de coordonnées p */
char* lecture(Point,char***);
/* [change_val_matrice(p,msg,matrice)] hange la valeur de la matrice de
 * coordonnées p par la valeur msg */
void change_val_matrice(Point,char*, char***);
/* [affichmat(mat,sizeLAB)] affiche sur la sortie standard la matrice
 * de taille sizeLAB */
void affichmat(char***,Point);


/******************************************************************************/
/* Fonctions de base d'un tableau dynamique de pointeurs                      */
/******************************************************************************/

/* [init_tab(size)] initialise un tableau de pointeurs à une dimension */
Point** init_tab(Point size);
/* [add_tab pt(tab,taille)] ajoute un point à un tableau de pointeurs */
Point** add_tab(Point pt, Point** tab, int taille);


/******************************************************************************/
/* Fonctions de base de la structure variables                                */
/******************************************************************************/

/* [init_listevars()] initialise une structure de type variables */
variables init_listevars();
/* [find_var(var,listevars)] vérifie si l'identifiant var est dans listevars */
int find_var(char*,variables);
/* [value_var(var,listevars)] renvoie la valeur de l'identifiant var
 * dans listevars */
int value_var(char*,variables);
/* [create_modif_var(nom,val,listevars)] modifie la valeur associée à l'identifiant :
 * - Si "nom" fait partie de la liste des variables, alors la variable est
 *   directement modifiée
 * - Sinon rajoute le nom de la variable et sa valeur dans listevars
     si la taille de listevars le permet */
variables create_modif_var(char*,int,variables);
/* [increment_var(var,listevars)] incrémente la valeur de l'identifiant :
 * - si l'identifiant var existe dans listevars, il incrémente la valeur existante
 * - sinon, il crée l'identifiant et l'initialise à 1
 * Utile pour listCOUNT */
variables increment_var(char*,variables);


/******************************************************************************/
/* Fonctions de base de la structure varpoint                                 */
/******************************************************************************/

/* [init_listevarpoint()] initialise une structure de type varpoint */
varpoint init_listevarpoint();
/* [find_varpoint(var,list)] vérifie si l'identifiant var est dans list */
int find_varpoint(char*,varpoint);
/* [value_varpoint(var,list)] renvoie la valeur de l'identifiant var dans list */
Point value_varpoint(char*,varpoint);
/* [create_modif_varpoint(nom,val,list)] modifie la valeur val associée à
 * l'identifiant :
 * - Si "nom" fait partie de la liste des variables, alors la variable est
 *   directement modifiée
 * - Sinon rajoute le nom de la variable et sa valeur dans list
     si la taille de list le permet */
varpoint create_modif_varpoint(char*,Point,varpoint);


/******************************************************************************/
/* Fonctions utiles au fichier labgen.y                                       */
/******************************************************************************/

/* Conventions finales sur la récupération d'informations du labyrinthe :
 * - Point size : taille du labyrinthe
 * - Point in : unique point d'entrée IN du labyrinthe
 * - Point** listOUT : tableau de pointeurs des points de sortie OUT
 * - Point** listWALL : tableau de pointeurs des points représentant les murs WALL
 * - Point** listTDV : tableau de pointeurs des points représentant
                       les trous de vers TDV
 * - varpoint** listPM : tableau de pointeurs sur un varpoint
 *                       -> pour listPM, on a n éléments de tableaux
 *                       -> pour chaque élément, on a un pointeur pointant vers
 *                          une structure de type varpoint
 *                       -> Cette structure varpoint a pour :
 *                          - identifiant : la direction (N,W,S,NE,...)
 *                          - valeur : un pointeur sur le point correspondant
 *                                     à la direction indiquée
 *                       -> Le point de départ est spécifié dans la structure
 *                          varpoint et a pour identifiant "DEP"
 * - variables listCOUNT : compte le nombre de points associés à chaque
 *                         identifiant SIZE, IN, OUT, WALL, TDV, PM
 *                         Par exemple : 1 point pour SIZE et pour IN
 *                                       3 points pour OUT  */

/* [show(size, in, listOUT, listWALL, listTDV, listPM, listCOUNT)] fait afficher
 * la matrice correspondant aux informations données sur le labyrinthe */
void show(Point, Point, Point**, Point**, Point**, varpoint**, variables);

int RS1(Point);
int RS2_count(int);
int RS3_count(int);
int RS2_3_check(Point E, Point* listeS, int sizeS, Point* listeETV, int sizeETV);
int RS4(Point E, Point* S, int nb_S, Point size);
int RS5();  /* A remodifier */
int RS6(Point* listeETV, int sizeETV, Point sizeLAB);
int RS7(Point* listePM, int sizePM, Point sizeLAB);
int RS8_PTA(Point* listePTA, int sizePTA, Point sizeLAB);
int RS8_PTD(Point PTD, Point* wallptd, int sizeW, int* wallr, Point sizeLAB);
int RS8_R(Point PTA1, Point PTA2, Point sizeLAB);

#endif
