#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "labgen.h"

/******************************************************************************/
/* Règles sémantiques :
 * Retourne 1 quand c'est correct
 * Retourne 0 sinon */
/******************************************************************************/

/* RS1 : Le labyrinthe doit avoir au moins 2 lignes et au moins 2 colonnes */
int RS1(Point size){
  if (size.c >= 1 && size.l >= 1)
    return 1;
  printf("RS1 : Le labyrinthe doit avoir au moins 2 lignes et au moins 2 colonnes.\n");
  return 0;
}

/* RS2_count : Le labyrinthe doit avoir une et une seule entrée */
int RS2_count(int countE){
  if (countE == 1)
    return 1;
  printf("RS2 : Le labyrinthe doit avoir une et une seule entrée.\n");
  return 0;
}


/* RS3_count : Le labyrinthe doit avoir au moins une sortie */
int RS3_count(int countS){
  if (countS >= 1)
    return 1;
  printf("RS3 : Le labyrinthe doit avoir au moins une sortie.\n");
  return 0;
}



/* RS2_3_check : L'entrée ne peut être ni une sortie ni une entrée de trou de vers
 *               Les sorties ne peuvent être une entrée d'un trou de vers
 * E est le point d'entrée
 * listeS est la liste de toutes les sorties
 * sizeS est la taille de listeS
 * listeETV est la liste de toutes les entrées de trous de vers
 * liste est la taille de listeETV */
int RS2_3_check(Point E, Point* listeS, int sizeS, Point* listeETV, int sizeETV){
  int i,j;
  int Ec = E.c;
  int El = E.l;
  Point tmpS, tmpETV;

  /* Vérifie que l'entrée ne correspond à aucune des entrées de trous de vers */
  for (j=0; j<sizeETV; j++){
    tmpETV = listeETV[j];
    if (Ec == tmpETV.c && El == tmpETV.l)
      return 0;
  }

  /* Vérifie que l'entrée ne correspond à aucune des sorties */
  for (i=0; i<sizeS; i++){
    tmpS = listeS[i];
    if (Ec == tmpS.c && El == tmpS.l)
      return 0;

    /*Vérifie que la sortie ne correspond pas à une entrée de vers*/
    for (j=0; j<sizeETV; j++){
      tmpETV = listeETV[j];
      if (tmpETV.c == tmpS.c && tmpETV.l == tmpS.l)
        return 0;
    }
  }

  return 1;
}


/* RS4 : L'entrée et les sorties du labyrinthe doivent se situer sur la périphérie
 * du labyrinthe
 * E est le point d'entrée
 * S est un tableau dynamique de l'ensemble des sorties
 * nb_S est le nombre de sorties
 * size est un point donnant la taille du tableau */
int RS4(Point E, Point* S, int nb_S, Point size){
   int i;
   Point tmp;
   int sizec = size.c;
   int sizel = size.l;

   if (S == NULL){
     fprintf(stderr,"RS4 : La liste des sorties est non initialisée.\n");
     exit(1);
   }

   else if (nb_S == 0){
     printf("RS4 : La liste des sorties est vide.\n");
     return 0;
   }

   /* Vérification pour l'entrée */
   else if (E.c != 0 && E.l != 0 && E.c != sizec && E.l != sizel){
      return 0;
   }

   /* Vérification pour toutes les sorties */
   else {
     for (i=0 ; i<nb_S ; i++){
       tmp = S[i];
       if (tmp.c != 0 && tmp.l != 0 && tmp.c != sizec && tmp.l != sizel){
          return 0;
       }
     }
   }
   return 1;
 }


 /* RS5 : Tant que la taille du labyrinthe n'est pas définie, il est interdit :
  * - de définir l'entrée ou des sorties
  * - d'utiliser des instructions de tracé
  * - De définir des portes magiques ou des trous de vers */
int RS5(){
  /* A faire */
  return 0;
}


/* Cette fonction est une fonction auxiliaire qui permet de vérifier si
 * l'ensemble des points de la liste sont dans le labyrinthe.
 * - liste est la liste de tous les points
 * - sizeL est la longueur de la liste
 * - sizeLAB est la taile du labyrinthe */
int danslelaby(Point* liste, int sizeL, Point sizeLAB, char* msg){
  int i;
  Point tmp;
  int sizec = sizeLAB.c;
  int sizel = sizeLAB.l;

  if (sizeL == 0){
    printf("La liste des %s est vide.\n",msg);
    return 1;
    /* Vrai car le labyrinthe peut ne pas contenir d'ETV, de PM ou de murs */
  }

  for (i=0; i<sizeL; i++){
    tmp = liste[i];
    if (tmp.c < 0 || tmp.c > sizec || tmp.l < 0 || tmp.l > sizel)
      return 0;
  }
  return 1;
}


/* RS6 : L'entrée et la sortie des trous de vers doivent être dans le
 *       labyrinthe
 * listeETV contient la liste de toutes les entrées et sorties de vers
 * sizeETV est la longueur de listeETV
 * sizeLAB est la taille du labyrinthe */
int RS6(Point* listeETV, int sizeETV, Point sizeLAB){
  return danslelaby(listeETV,sizeETV,sizeLAB,"entrées de vers");
}


/* RS7 : L'entrée et les sorties des portes magiques doivent être dans le
         labyrinthe
 * listePM est la liste des entrées et sorties des portes magiques
 * sizePM est la longueur de listePM
 * sizeLAB est la taille du labyrinthe */
 int RS7(Point* listePM, int sizePM, Point sizeLAB){
   return danslelaby(listePM,sizePM,sizeLAB,"portes magiques");
 }


/* RS8_PTA : Pour les tracés de murs les points définis par l'instruction
 *           WALL PTA doivent être dans le labyrinthe
 * listePTA est la liste de tous les murs définis par PTA
 * sizePTA est la longueur de listePTA
 * sizeLAB est la taille du labyrinthe */
int RS8_PTA(Point* listePTA, int sizePTA, Point sizeLAB){
  return danslelaby(listePTA,sizePTA,sizeLAB,"murs définis par PTA");
}


/* RS8_PTD : Pour les tracés de murs les points définis par l'instruction
 *           WALL PTD doivent être dans le labyrinthe
 * PTD est le point de départ
 * wallptd est la liste des points donnant le déplacement
 * wallr est la liste d'entiers, éventuellement nuls, donnant le nombre de
 *        de déplacement pour chaque point wallptd.
 *        On suppose que les listes wallptd et wallr ont la même taille
 * sizeW donne la longueur de la liste wallptd
 * sizeLAB est la taille du labyrinthe */
int RS8_PTD(Point PTD, Point* wallptd, int sizeW, int* wallr, Point sizeLAB){
  int sizec = sizeLAB.c;
  int sizel = sizeLAB.l;
  int i,j;
  Point res;
  Point tmpW;
  int tmpR;

  /* On vérifie que le point de départ est dans le labyrinthe */
  if (PTD.c < 0 || PTD.c > sizec || PTD.l < 0 || PTD.l > sizel)
    return 0;
  /* On vérifie que le déplacement ne nous fait pas sortir du labyrinthe */
  res = PTD;
  for (i=0; i<sizeW; i++){
    tmpW = wallptd[i];
    tmpR = wallr[i];
    res.c += tmpW.c;
    res.l += tmpW.l;
    if (res.c < 0 || res.c > sizec || res.l < 0 || res.l > sizel)
      return 0;
    if (tmpR > 1){
      for (j=1; j<tmpR; j++){ /*On le fait bien tmpR-1 fois car on a déjà fait 1 fois avant */
        res.c += tmpW.c;
        res.l += tmpW.l;
        if (res.c < 0 || res.c > sizec || res.l < 0 || res.l > sizel)
          return 0;
      }
    }
  }

  return 1;
}


/* RS8_R : Pour les tracés de murs les points définis par l'instruction WALL R
 *         doivent être dans le labyrinthe
 * PTA1 est le premier point définissant le mur, PTA2 le deuxième
 * sizeLAB est la taille du labyrinthe */
int RS8_R(Point PTA1, Point PTA2, Point sizeLAB){
  int sizec = sizeLAB.c;
  int sizel = sizeLAB.l;

  if (PTA1.c < 0 || PTA1.c > sizec || PTA1.l < 0 || PTA1.l > sizel)
    return 0;
  if (PTA2.c < 0 || PTA2.c > sizec || PTA2.l < 0 || PTA2.l > sizel)
    return 0;
  return 1;
}

int RS8_FOR(){
  /* A faire */
  return 0;
}

int RS8_1F(){
  /* A faire */
  return 0;
}

int RS9(){
  /* A faire */
  return 0;
}

int RS10(){
  /* A faire */
  return 0;
}

int RS11(){
  /* A faire */
  return 0;
}

/* Test des fonctions pour les règles sémantiques */
int main(){
  char*** matrice = create_matrice(5,6);
  Point size = new_point(5,6);

  Point entree1 = new_point(0,0);
  Point entree2 = new_point(5,6);
  Point entree3 = new_point(5,0);
  Point entree4 = new_point(0,5);

  change_val_matrice(entree1,"E",matrice);
  change_val_matrice(entree2,"E",matrice);
  change_val_matrice(entree3,"E",matrice);
  change_val_matrice(entree4,"E",matrice);

  Point* listeS = NULL;
  listeS = calloc(5,sizeof(Point));
  listeS[0] = new_point(5,5);
  listeS[1] = new_point(3,6);
  listeS[2] = new_point(0,0);
  listeS[3] = new_point(0,4);
  listeS[4] = new_point(1,6);

  Point* listeETV = NULL;
  listeETV = calloc(6,sizeof(Point));
  listeETV[0] = new_point(2,4);
  listeETV[1] = new_point(1,0);
  listeETV[2] = new_point(1,1);
  listeETV[3] = new_point(4,3);
  listeETV[4] = new_point(5,6);
  listeETV[5] = new_point(0,7);

  Point* listePM = NULL;
  listePM = calloc(6,sizeof(Point));
  listePM[0] = new_point(2,4);
  listePM[1] = new_point(1,0);
  listePM[2] = new_point(1,1);
  listePM[3] = new_point(4,3);
  listePM[4] = new_point(5,6);
  listePM[5] = new_point(0,7);

  Point* wallPTD = NULL;
  wallPTD = calloc(5,sizeof(Point));
  wallPTD[0] = new_point(1,-1);
  wallPTD[1] = new_point(0,1);
  wallPTD[2] = new_point(1,0);
  wallPTD[3] = new_point(1,1);
  wallPTD[4] = new_point(1,0);

  int* wallr = NULL;
  wallr = calloc(5,sizeof(int));
  wallr[0] = 0;
  wallr[1] = 3;
  wallr[2] = 0;
  wallr[3] = 0;
  wallr[4] = 2;


  change_val_matrice(new_point(5,5),"S",matrice);
  change_val_matrice(new_point(5,6),"S",matrice);
  change_val_matrice(new_point(0,0),"S",matrice);
  change_val_matrice(new_point(0,4),"S",matrice);
  change_val_matrice(new_point(1,6),"S",matrice);

  affichmat(matrice,size);

  printf("Test RS2_check - entree1 : %d [0]\n", RS2_3_check(entree1,listeS,5,listeETV,5));
  printf("Test RS2_check - entree2 : %d [0]\n", RS2_3_check(entree2,listeS,5,listeETV,5));
  printf("Test RS2_check - entree3 : %d [1]\n", RS2_3_check(entree3,listeS,5,listeETV,5));
  printf("Test RS2_check - entree4 : %d [1]\n", RS2_3_check(entree4,listeS,5,listeETV,5));
  printf("Test RS4 - entree1 : %d [1]\n", RS4(entree1,listeS,5,size));
  printf("Test RS4 - entree2 : %d [1]\n", RS4(entree2,listeS,5,size));
  printf("Test RS4 - entree3 : %d [1]\n", RS4(entree3,listeS,5,size));
  printf("Test RS4 - entree4 : %d [1]\n", RS4(entree4,listeS,5,size));
  printf("Test RS6 : %d [1]\n", RS6(listeETV,5,size));
  printf("Test RS6 : %d [0]\n", RS6(listeETV,6,size));
  printf("Test RS7 : %d [1]\n", RS6(listePM,5,size));
  printf("Test RS7 : %d [0]\n", RS6(listePM,6,size));
  printf("Test RS8_PTD : %d [1]\n",RS8_PTD(new_point(0,2),wallPTD,5,wallr,size));


  return 0;
}
